@@ -7,6 +7,8 @@
 #include <QLocale>
 #include <QTest>
 
+#include <unistd.h>
+
 #include "Formatter.h"
 #include "Unit.h"
 
@@ -53,12 +55,13 @@ private Q_SLOTS:
     {
         QTest::addColumn<int>("input");
         QTest::addColumn<QString>("output");
-        QTest::newRow("1 s") << 1 << QSL("0:01");
-        QTest::newRow("10 s") << 10 << QSL("0:10");
-        QTest::newRow("1 m") << 60 << QSL("1:00");
-        QTest::newRow("10m") << 60 * 10 << QSL("10:00");
-        QTest::newRow("1h") << 60 * 60 << QSL("60:00");
-        QTest::newRow("1h 1 m 1s") << (60 * 60) + 60 + 1 << QSL("61:01");
+        QTest::newRow("1 s") << 1 << QSL("0:00:01");
+        QTest::newRow("10 s") << 10 << QSL("0:00:10");
+        QTest::newRow("1 m") << 60 << QSL("0:01:00");
+        QTest::newRow("10m") << 60 * 10 << QSL("0:10:00");
+        QTest::newRow("1h") << 60 * 60 << QSL("1:00:00");
+        QTest::newRow("1h 1 m 1s") << (60 * 60) + 60 + 1 << QSL("1:01:01");
+        QTest::newRow("25h 0m 0s") << (25 * 3600) << QSL("25:00:00");
     }
 
     void testFormatTime()
@@ -68,6 +71,39 @@ private Q_SLOTS:
         auto formatted = KSysGuard::Formatter::formatValue(input, KSysGuard::UnitTime);
         QCOMPARE(formatted, output);
     }
+
+    void testFormatBootTimestamp_data()
+    {
+        QTest::addColumn<int>("input");
+        // This uses a list because we are comparing translated strings and those may
+        // end up slightly different in some cases.
+        QTest::addColumn<QStringList>("output");
+        QTest::newRow("1s ago") << 1 << QStringList{QSL("Just now")};
+        QTest::newRow("10s ago") << 10 << QStringList{QSL("Just now")};
+        QTest::newRow("1m ago") << 60 << QStringList{QSL("Just now")};
+        QTest::newRow("10m ago") << 600 << QStringList{QSL("10 minute(s) ago"), QSL("10 minutes ago")};
+        QTest::newRow("59m59s ago") << 3559 << QStringList{QSL("59 minute(s) ago"), QSL("59 minutes ago")};
+    }
+
+    void testFormatBootTimestamp()
+    {
+        QFETCH(int, input);
+        QFETCH(QStringList, output);
+
+        timespec tp;
+#ifdef Q_OS_LINUX
+        clock_gettime(CLOCK_BOOTTIME, &tp);
+#else
+        clock_gettime(CLOCK_MONOTONIC, &tp);
+#endif
+
+        const auto ticks = sysconf(_SC_CLK_TCK);
+
+        const QDateTime systemBootTime = QDateTime::currentDateTime().addSecs(-tp.tv_sec);
+        auto elapsed = QDateTime::currentDateTime().addSecs(-input);
+        auto formatted = KSysGuard::Formatter::formatValue(systemBootTime.secsTo(elapsed) * ticks, KSysGuard::UnitBootTimestamp);
+        QVERIFY(output.contains(formatted));
+    }
 };
 
 QTEST_MAIN(FormatterTest);
